@@ -3,11 +3,10 @@ import html
 import logging
 import re
 
-import mailbits
-
+from .exceptions import StandardViolationError
 from .message_base import MessageBase
 from .signed_attachment import SignedAttachment
-from .utils import inputToString
+from .utils import inputToString, unwrapMultipart
 
 
 logger = logging.getLogger(__name__)
@@ -68,59 +67,26 @@ class MessageSignedBase(MessageBase):
     def attachments(self) -> list:
         """
         Returns a list of all attachments.
+
+        :raises StandardViolationError: The standard for signed messages was
+            blatantly violated.
         """
         try:
             return self._sAttachments
         except AttributeError:
             atts = super().attachments
-            self._sAttachments = []
-            self._signedBody = None
-            self._signedHtmlBody = None
-            if len(atts) > 1:
-                logger.warning('Found more than one regular attachment when parsing signed message.')
-            elif len(atts) == 0:
-                logger.warning('Failed to access any attachments from the signed message.')
-                return self._sAttachments
 
-            try:
-                mainAttachment = next(att for att in atts if hasattr(att, 'getFilename') and att.getFilename() == 'smime.p7m')
-            except StopIteration:
-                logger.warning('Failed to find signed attachment.')
-                return self._sAttachments
+            if len(atts) != 1:
+                raise StandardViolationError('Signed messages without exactly 1 (regular) attachment constitue a violation of the standard.')
 
-            # If we are here, we should have the attachment. So now we need to
-            # try to parse and unwrap the data.
-            toParse = [mailbits.email2dict(email.message_from_bytes(mainAttachment.data))]
-            output = []
+            # We need to unwrap the multipart stream.
+            unwrapped = unwrapMultipart(atts[0].data)
 
-            while len(toParse) != 0:
-                parsing = toParse.pop(0)
-                for part in parsing['content']:
-                    # If it is multipart, push it to the toParse list, otherwise add it
-                    # to the output.
-                    if part['headers']['content-type']['content_type'].startswith('multipart'):
-                        toParse.append(part)
-                    else:
-                        output.append(part)
-
-            # At this point, `output` has out parts.
-            for part in output:
-                # Get the mime type.
-                mime = part['headers']['content-type']['content_type']
-
-                # Now try to grab a name. If it doesn't exist, we make one.
-                try:
-                    name = part['headers']['content-type']['params']['name']
-                except KeyError:
-                    if mime == 'text/plain':
-                        self._signedBody = part['content']
-                        continue
-                    elif mime == 'text/html':
-                        self._signedHtmlBody = part['content']
-                        continue
-                    else:
-                        name = 'unknown.bin'
-                self._sAttachments.append(self.__signedAttachmentClass(self, part['content'], name, mime))
+            # Now store everything where it needs to be and make the
+            # attachments.
+            self._sAttachments = [self.__signedAttachmentClass(self, **att) for att in unwrapped['attachments']]
+            self._signedBody = unwrapped['plain_body']
+            self._signedHtmlBody = unwrapped['html_body']
 
             return self._sAttachments
 
