@@ -4,8 +4,11 @@ Utility functions of extract_msg.
 
 import argparse
 import codecs
+import collections
 import copy
 import datetime
+import email.message
+import email.policy
 import glob
 import json
 import logging
@@ -22,8 +25,10 @@ import bs4
 import tzlocal
 
 from html import escape as htmlEscape
+from typing import Dict, List, Tuple, Union
 
 from . import constants
+from .enums import AttachmentType
 from .exceptions import BadHtmlError, ConversionError, IncompatibleOptionsError, InvaildPropertyIdError, UnknownCodepageError, UnknownTypeError, UnrecognizedMSGTypeError, UnsupportedMSGTypeError
 
 
@@ -45,6 +50,7 @@ def addNumToDir(dirName : pathlib.Path) -> pathlib.Path:
             pass
     return None
 
+
 def addNumToZipDir(dirName : pathlib.Path, _zip):
     """
     Attempt to create the directory with a '(n)' appended.
@@ -56,6 +62,7 @@ def addNumToZipDir(dirName : pathlib.Path, _zip):
             return newDirName
     return None
 
+
 def bitwiseAdjust(inp : int, mask : int) -> int:
     """
     Uses a given mask to adjust the location of bits after an operation like
@@ -74,6 +81,7 @@ def bitwiseAdjust(inp : int, mask : int) -> int:
         raise ValueError('Mask MUST be greater than 0')
     return inp >> bin(mask)[::-1].index('1')
 
+
 def bitwiseAdjustedAnd(inp : int, mask : int) -> int:
     """
     Preforms the bitwise AND operation between :param inp: and :param mask: and
@@ -85,6 +93,7 @@ def bitwiseAdjustedAnd(inp : int, mask : int) -> int:
         raise ValueError('Mask MUST be greater than 0')
     return (inp & mask) >> bin(mask)[::-1].index('1')
 
+
 def bytesToGuid(bytesInput : bytes) -> str:
     """
     Converts a bytes instance to a GUID.
@@ -92,6 +101,7 @@ def bytesToGuid(bytesInput : bytes) -> str:
     guidVals = constants.ST_GUID.unpack(bytesInput)
     return f'{{{guidVals[0]:08X}-{guidVals[1]:04X}-{guidVals[2]:04X}-{guidVals[3][:2].hex().upper()}-{guidVals[3][2:].hex().upper()}}}'
 
+
 def ceilDiv(n : int, d : int) -> int:
     """
     Returns the int from the ceil division of n / d.
@@ -102,6 +112,7 @@ def ceilDiv(n : int, d : int) -> int:
     """
     return -(n // -d)
 
+
 def createZipOpen(func):
     """
     Creates a wrapper for the open function of a ZipFile that will automatically
@@ -115,6 +126,7 @@ def createZipOpen(func):
 
     return _open
 
+
 def divide(string, length : int) -> list:
     """
     Divides a string into multiple substrings of equal length. If there is not
@@ -135,6 +147,7 @@ def divide(string, length : int) -> list:
     """
     return [string[length * x:length * (x + 1)] for x in range(int(ceilDiv(len(string), length)))]
 
+
 def findWk(path = None):
     """
     Attempt to find the path of the wkhtmltopdf executable. If :param path: is
@@ -158,12 +171,14 @@ def findWk(path = None):
 
     raise ExecutableNotFound('Could not find wkhtmltopdf.')
 
+
 def fromTimeStamp(stamp) -> datetime.datetime:
     """
     Returns a datetime from the UTC timestamp given the current timezone.
     """
     return datetime.datetime.fromtimestamp(stamp, tzlocal.get_localzone())
 
+
 def getCommandArgs(args):
     """
     Parse command-line arguments.
@@ -308,6 +323,7 @@ def getCommandArgs(args):
 
     return options
 
+
 def getEncodingName(codepage : int) -> str:
     """
     Returns the name of the encoding with the specified codepage.
@@ -323,15 +339,18 @@ def getEncodingName(codepage : int) -> str:
     except LookupError:
         raise UnsupportedEncodingError(f'The codepage {codepage} ({constants.CODE_PAGES[codepage]}) is not currently supported by your version of Python.')
 
+
 def getFullClassName(inp):
     return inp.__class__.__module__ + '.' + inp.__class__.__name__
 
+
 def hasLen(obj) -> bool:
     """
     Checks if :param obj: has a __len__ attribute.
     """
     return hasattr(obj, '__len__')
 
+
 def injectHtmlHeader(msgFile, prepared : bool = False) -> bytes:
     """
     Returns the HTML body from the MSG file (will check that it has one) with
@@ -357,7 +376,6 @@ def injectHtmlHeader(msgFile, prepared : bool = False) -> bytes:
         # If the body is not valid or not found, raise an AttributeError.
         if not body:
             raise AttributeError('Cannot find a prepared HTML body to inject into.')
-
     else:
         body = msgFile.htmlBody
 
@@ -453,6 +471,7 @@ def injectHtmlHeader(msgFile, prepared : bool = False) -> bytes:
     # Use the previously defined function to inject the HTML header.
     return constants.RE_HTML_BODY_START.sub(replace, body, 1)
 
+
 def injectRtfHeader(msgFile) -> bytes:
     """
     Returns the RTF body from the MSG file (will check that it has one) with the
@@ -533,6 +552,7 @@ def injectRtfHeader(msgFile) -> bytes:
 
     raise RuntimeError('All injection attempts failed. Please report this to the developer.')
 
+
 def inputToBytes(stringInputVar, encoding) -> bytes:
     """
     Converts the input into bytes.
@@ -548,6 +568,7 @@ def inputToBytes(stringInputVar, encoding) -> bytes:
     else:
         raise ConversionError('Cannot convert to bytes.')
 
+
 def inputToMsgpath(inp) -> list:
     """
     Converts the input into an msg path.
@@ -557,6 +578,7 @@ def inputToMsgpath(inp) -> list:
     ret = inputToString(inp, 'utf-8').replace('\\', '/').split('/')
     return ret if ret[0] != '' else []
 
+
 def inputToString(bytesInputVar, encoding) -> str:
     """
     Converts the input into a string.
@@ -572,6 +594,7 @@ def inputToString(bytesInputVar, encoding) -> str:
     else:
         raise ConversionError('Cannot convert to str type.')
 
+
 def isEncapsulatedRtf(inp : bytes) -> bool:
     """
     Currently the destection is made to be *extremly* basic, but this will work
@@ -580,12 +603,14 @@ def isEncapsulatedRtf(inp : bytes) -> bool:
     """
     return b'\\fromhtml' in inp
 
+
 def isEmptyString(inp : str) -> bool:
     """
     Returns true if the input is None or is an Empty string.
     """
     return (inp == '' or inp is None)
 
+
 def knownMsgClass(classType : str) -> bool:
     """
     Checks if the specified class type is recognized by the module. Usually used
@@ -601,12 +626,14 @@ def knownMsgClass(classType : str) -> bool:
 
     return False
 
+
 def filetimeToUtc(inp : int) -> float:
     """
     Converts a FILETIME into a unix timestamp.
     """
     return (inp - 116444736000000000) / 10000000.0
 
+
 def msgpathToString(inp) -> str:
     """
     Converts an msgpath (one of the internal paths inside an msg file) into a
@@ -619,6 +646,7 @@ def msgpathToString(inp) -> str:
     inp.replace('\\', '/')
     return inp
 
+
 def openMsg(path, **kwargs):
     """
     Function to automatically open an MSG file and detect what type it is.
@@ -691,6 +719,31 @@ def openMsg(path, **kwargs):
         logger.error(f'Could not recognize msg class type "{msg.classType}". This most likely means it hasn\'t been implemented yet, and you should ask the developers to add support for it.')
         return msg
 
+
+def openMsgBulk(path, **kwargs) -> Union[List, Tuple]:
+    """
+    Takes the same arguments as openMsg, but opens a collection of msg files
+    based on a wild card. Returns a list if successful, otherwise returns a
+    tuple.
+
+    :param ignoreFailures: If this is True, will return a list of all successful
+        files, ignoring any failures. Otherwise, will close all that
+        successfully opened, and return a tuple of the exception and the path of
+        the file that failed.
+    """
+    files = []
+    for x in glob.glob(path):
+        try:
+            files.append(openMsg(x, **kwargs))
+        except Exception as e:
+            if not kwargs.get('ignoreFailures', False):
+                for msg in files:
+                    msg.close()
+                return (e, x)
+
+    return files
+
+
 def parseType(_type : int, stream, encoding, extras):
     """
     Converts the data in :param stream: to a much more accurate type, specified
@@ -825,6 +878,7 @@ def parseType(_type : int, stream, encoding, extras):
             raise NotImplementedError(f'Parsing for type {_type} has not yet been implmented. If you need this type, please create a new issue labeled "NotImplementedError: parseType {_type}".')
     return value
 
+
 def prepareFilename(filename) -> str:
     """
     Adjusts :param filename: so that it can succesfully be used as an actual
@@ -833,6 +887,7 @@ def prepareFilename(filename) -> str:
     # I would use re here, but it tested to be slightly slower than this.
     return ''.join(i for i in filename if i not in r'\/:*?"<>|' + '\x00')
 
+
 def properHex(inp, length : int = 0) -> str:
     """
     Takes in various input types and converts them into a hex string whose
@@ -849,12 +904,14 @@ def properHex(inp, length : int = 0) -> str:
         a = '0' + a
     return a.rjust(length, '0').upper()
 
+
 def roundUp(inp : int, mult : int) -> int:
     """
     Rounds :param inp: up to the nearest multiple of :param mult:.
     """
     return inp + (mult - inp) % mult
 
+
 def rtfSanitizeHtml(inp : str) -> str:
     """
     Sanitizes input to an RTF stream that has encapsulated HTML.
@@ -885,6 +942,7 @@ def rtfSanitizeHtml(inp : str) -> str:
 
     return output
 
+
 def rtfSanitizePlain(inp : str) -> str:
     """
     Sanitizes input to a plain RTF stream.
@@ -907,6 +965,7 @@ def rtfSanitizePlain(inp : str) -> str:
 
     return output
 
+
 def setupLogging(defaultPath = None, defaultLevel = logging.WARN, logfile = None, enableFileLogging : bool = False,
                   env_key = 'EXTRACT_MSG_LOG_CFG') -> bool:
     """
@@ -985,6 +1044,192 @@ def setupLogging(defaultPath = None, defaultLevel = logging.WARN, logfile = None
     logging.getLogger().setLevel(defaultLevel)
     return True
 
+
+def unwrapMsg(msg : "MSGFile") -> Dict:
+    """
+    Takes a recursive message-attachment structure and unwraps it into a linear
+    dictionary for easy iteration. Dictionary contains 4 keys: "attachments" for
+    main message attachments, not including embedded MSG files, "embedded" for
+    attachments representing embedded MSG files, "msg" for all MSG files
+    (including the original in the first index), and "raw_attachments" for raw
+    attachments from signed messages.
+    """
+    from .message_signed_base import MessageSignedBase
+
+    # Here is where we store main attachments.
+    attachments = []
+    # Here is where we are going to store embedded msg files.
+    msgFiles = [msg]
+    # Here is where we store embedded attachments.
+    embedded = []
+    # Here is where we store raw attachments from signed messages.
+    raw = []
+
+    # Normally we would need a recursive function to unwrap a recursive
+    # structure like the message-attachment structure. Essentially, a function
+    # that calls itself. Here, I have designed code capable of circumventing
+    # this to do it in a single function, which is a lot more efficient and
+    # safer. That is why we store the `toProcess` and use a while loop
+    # surrounding a for loop. The for loop would be the main body of the
+    # function, while the append to toProcess would be the recursive call.
+    toProcess = collections.deque((msg,))
+
+    while len(toProcess) > 0:
+        # Remove the last item from the list of things to process, and store it
+        # in `currentItem`. We will be processing it in the for loop.
+        currentItem = toProcess.popleft()
+        # iterate through the attachments and
+        for att in currentItem.attachments:
+            # If it is a regular attachment, add it to the list. Otherwise, add
+            # it to be processed
+            if att.type in (AttachmentType.DATA, AttachmentType.SIGNED):
+                attachments.append(att)
+            elif att.type == AttachmentType.MSG:
+                # Here we do two things. The first is we store it to the output
+                # so we can return it. The second is we add it to the processing
+                # list. The reason this is two steps is because we need to be
+                # able to remove items from the processing list, but can't
+                # do that from the output.
+                embedded.append(att)
+                msgFiles.append(att.data)
+                toProcess.append(att.data)
+        if isinstance(currentItem, MessageSignedBase):
+            raw += currentItem._rawAttachments
+
+    return {
+        'attachments': attachments,
+        'embedded': embedded,
+        'msg': msgFiles,
+        'raw_attachments': raw,
+    }
+
+
+def unwrapMultipart(mp : Union[bytes, str, email.message.Message]) -> Dict:
+    """
+    Unwraps a recursive multipart structure into a dictionary of linear lists.
+    Similar to unwrapMsg, but for multipart. Dictionary contains 3 keys:
+    "attachments" which contains a list of dicts containing processed attachment
+    data as well as the Message instance associated with it, "plain_body" which
+    contains the plain text body, and "html_body" which contains the HTML body.
+
+    For clarification, each instance of processed attachment data is a dict
+    with keys identical to the args used for the SignedAttachment constructor.
+    This makes it easy to expand for use in constructing a SignedAttachment. The
+    only argument missing is "msg" to ensure this function will not require one.
+
+    :param mp: The bytes that make up a multipart, the string that makes up a
+        multipart, or a Message instance from the email module created from the
+        multipart to unwrap. If providing a Message instance, prefer it to be an
+        instance of EmailMessage. If you are doing so, make sure it's policy is
+        default.
+    """
+    # In the event we are generating it, these are the kwargs to use.
+    genKwargs = {
+        '_class': email.message.EmailMessage,
+        'policy': email.policy.default,
+    }
+    # Convert our input into something usable.
+    if isinstance(mp, email.message.EmailMessage):
+        if mp.policy == email.policy.default:
+            mpMessage = mp
+        else:
+            mpMessage = email.message_from_bytes(mp.as_bytes(), **genKwargs)
+    elif isinstance(mp, email.message.Message):
+        mpMessage = email.message_from_bytes(mp.as_bytes(), **genKwargs)
+    elif isinstance(mp, bytes):
+        mpMessage = email.message_from_bytes(mp, **genKwargs)
+    elif isinstance(mp, str):
+        mpMessage = email.message_from_string(mp, **genKwargs)
+    else:
+        raise TypeError(f'Unsupported type "{type(mp)}" provided to unwrapMultipart.')
+
+    # Okay, now that we have it in a useable form, let's do the most basic
+    # unwrapping possible. Once the most basic unwrapping is done, we can
+    # actually process the data. For this, we only care if the section is
+    # multipart or not. If it is, it get's unwrapped too.
+    #
+    # In case you are curious, this is effectively doing a breadth first
+    # traversal of the tree.
+    dataNodes = []
+
+    toProcess = collections.deque((mpMessage,))
+    # I do know about the walk method, but it might *also* walk embedded
+    # messages which we very much don't want.
+    while len(toProcess) > 0:
+        currentItem = toProcess.popleft()
+        # 'multipart' indicates that it shouldn't contain any data itself, just
+        # other nodes to go through.
+        if currentItem.get_content_maintype() == 'multipart':
+            payload = currentItem.get_payload()
+            # For multipart, the payload should be a list, but handle it not
+            # being one.
+            if isinstance(payload, list):
+                toProcess.extend(payload)
+            else:
+                logging.warn('Found multipart node that did not return a list. Appending as a data node.')
+                dataNodes.append(currentItem)
+        else:
+            # The opposite is *not* true. If it's not multipart, always add as a
+            # data node.
+            dataNodes.append(currentItem)
+
+    # At this point, all of our nodes should have processed and we should now
+    # have data nodes. Now let's process them. For anything that was parsed as
+    # a message, we actually want to get it's raw bytes back so it can be saved.
+    # If they user wants to process that message in some way, they can do it
+    # themself.
+    attachments = []
+    plainBody = None
+    htmlBody = None
+
+    for node in dataNodes:
+        # Let's setup our attachment we are going to use.
+        attachment = {
+            'data': None,
+            'name': node.get_filename(),
+            'mimetype': node.get_content_type(),
+            'node': node,
+        }
+
+        # Finally, we need to get the data. As we need to ensure it is bytes,
+        # we may have to do some special processing.
+        data = node.get_content()
+        if isinstance(data, bytes):
+            # If the data is bytes, we are perfectly good.
+            pass
+        elif isinstance(data, email.message.Message):
+            # If it is a message, get it's bytes directly.
+            data = data.as_bytes()
+        elif isinstance(data, str):
+            # If it is a string, let's reverse encode it where possible.
+            # First thing we want to check is if we can find the encoding type.
+            # If we can, use that to reverse the process. Otherwise use utf-8.
+            data = data.encode(node.get_content_charset('utf-8'))
+        else:
+            # We throw an exception to describe the problem if we can't reverse
+            # the problem.
+            raise TypeError(f'Attempted to get bytes for attachment, but could not convert {type(data)} to bytes.')
+
+        attachment['data'] = data
+
+        # Now for the fun part, figuring out if we actually have an attachment.
+        if attachment['name']:
+            attachments.append(attachment)
+        elif attachment['mimetype'] == 'text/plain':
+            if plainBody:
+                logger.warn('Found multiple candidates for plain text body.')
+            plainBody = data
+        elif attachment['mimetype'] == 'text/html':
+            if htmlBody:
+                logger.warn('Found multiple candidates for HTML body.')
+            htmlBody = data
+
+    return {
+        'attachments': attachments,
+        'plain_body': plainBody,
+        'html_body': htmlBody,
+    }
+
 def validateHtml(html : bytes) -> bool:
     """
     Checks whether the HTML is considered valid. To be valid, the HTML must, at
@@ -995,6 +1240,7 @@ def validateHtml(html : bytes) -> bool:
         return False
     return True
 
+
 def verifyPropertyId(id : str) -> None:
     """
     Determines whether a property ID is valid for vertain functions. Property
@@ -1014,6 +1260,7 @@ def verifyPropertyId(id : str) -> None:
         except ValueError:
             raise InvaildPropertyIdError('ID was not a 4 digit hexadecimal string')
 
+
 def verifyType(_type):
     """
     Verifies that the type is valid. Raises an exception if it is not.
@@ -1024,5 +1271,6 @@ def verifyType(_type):
         if (_type not in constants.VARIABLE_LENGTH_PROPS_STRING) and (_type not in constants.FIXED_LENGTH_PROPS_STRING):
             raise UnknownTypeError(f'Unknown type {_type}.')
 
+
 def windowsUnicode(string):
     return str(string, 'utf-16-le') if string is not None else None
